@@ -1,5 +1,6 @@
 /* epicon_uty.c epicon utitlity program
-epicon is Copyright Isamu.Yamauchi 2002-2010.
+epicon is Copyright Isamu.Yamauchi 2002-2011.
+o 2011.10.16 Enhanced file locking, changed the function msleep.
 o 2010.11.19 warning: ignoring return value of ‘write’, declared with attribute warn_unused_result. clean up.
 o 2009.10.31 add -q option quiet mode, clean up.
 o 2009.6.8 delete CBAUD IUCLC OLCUC
@@ -79,56 +80,82 @@ char *c_result;                        /* tmp char result */
 FILE *f_result;                        /* tmp FILE result */
 com_port_use_check(char *com)
 {
-  FILE * fd;
   char file[128];
+  char buf[16];
   char * p;
-  int i;
-  fd = fopen(VAR_LOCK, "r");
-  if( fd == 0 ) return 1;
-  fclose(fd);
+  struct flock fl;
+  int fd;
+  fl.l_type   = F_RDLCK;  /* F_RDLCK, F_WRLCK, F_UNLCK    */
+  fl.l_whence = SEEK_SET; /* SEEK_SET, SEEK_CUR, SEEK_END */
+  fl.l_start  = 0;        /* Offset from l_whence         */
+  fl.l_len    = 0;        /* length, 0 = to EOF           */
+  fl.l_pid    = getpid(); /* our PID                      */
+  fd = open(VAR_LOCK, O_RDONLY);
+  if( fd < 0 ) return 1;
+  close(fd);
   p = strrchr(com, '/');
   strcpy(file, TTY_LOCK);
-  if( p ) strcat(file, p+1);
+  if( p ) strcat(file, p + 1);
   else  {
     strcat(file, com);
   }
-  for(p=file+21; *p; p++) if( *p>='A' && *p<='Z' ) *p = *p-'A'+'a';
-  fd = fopen(file, "r");
-  if( fd )  {
-    t_result = fscanf(fd, "%d", &i);
-    fclose(fd);
-    if( kill(i, 0) == 0 ) return 0;
-    if( unlink(file) == -1 ) return 0;
+  for(p = file + 21; *p; p++) if( *p >= 'A' && *p <= 'Z' ) *p = *p - 'A' + 'a';
+  if ((fd = open(file, O_RDONLY)) > 0 ) return 0;
+  if ((fd = open(file, O_CREAT | O_RDWR)) < 0 ) perror("open");
+  if (fcntl(fd, F_SETLK, &fl) < 0) {
+    perror("fcntl");
+    return 0;
   }
-  fd = fopen(file, "w");
-  fprintf(fd, "%10d\n", getpid()); 
-  fclose(fd);
+  sprintf(buf, "%d\n", getpid()); 
+  write(fd, buf, strlen(buf));
+  fl.l_type = F_UNLCK;
+  if (fcntl(fd, F_SETLK, &fl) < 0) {
+    perror("fcntl");
+    return 0;
+  }
+  close(fd);
   return 1;
 }
 
 com_port_unuse(char *com)
 {
-  FILE * fd;
   char file[128];
+  char buf[16];
   char * p;
-  int i;
-  fd = fopen(VAR_LOCK, "r");
-  if( fd == 0 ) return;
-  fclose(fd);
+  struct flock fl;
+  int fd,i;
+  fl.l_type   = F_RDLCK;  /* F_RDLCK, F_WRLCK, F_UNLCK    */
+  fl.l_whence = SEEK_SET; /* SEEK_SET, SEEK_CUR, SEEK_END */
+  fl.l_start  = 0;        /* Offset from l_whence         */
+  fl.l_len    = 0;        /* length, 0 = to EOF           */
+  fl.l_pid    = getpid(); /* our PID                      */
+  fd = open(VAR_LOCK, O_RDONLY);
+  if( fd < 0 ) return 1;
+  close(fd);
   p = strrchr(com, '/');
   strcpy(file, TTY_LOCK);
-  if(p) strcat(file,p+1);
+  if( p ) strcat(file, p + 1);
   else  {
-           strcat(file,com);
+    strcat(file, com);
   }
-  for(p=file+21; *p; p++) if( *p>='A' && *p<='Z' ) *p = *p-'A'+'a';
-  fd = fopen(file, "r");
-  if( fd )  {
-    t_result = fscanf(fd, "%d", &i);
-    fclose(fd);
-    if( i == getpid() || kill(i, 0) != 0 )
-    unlink(file);
+  for(p = file + 21; *p; p++) if( *p >= 'A' && *p <= 'Z' ) *p = *p - 'A' + 'a';
+  if ((fd = open(file, O_RDONLY)) < 0 ) {
+    return 1;
   }
+  if (fcntl(fd, F_SETLK, &fl) < 0) {
+    perror("fcntl");
+    return 0;
+  }
+  read(fd, buf, sizeof(buf));
+  i = atoi(buf);
+  fl.l_type = F_UNLCK;
+  if (fcntl(fd, F_SETLK, &fl) < 0) {
+    perror("fcntl");
+    return 0;
+  }
+  close(fd);
+  if ( i == fl.l_pid || getppid() == 1 ) unlink(file);
+  return 1;
 }
 
 shell_process()  {
@@ -771,9 +798,12 @@ if ( !Quiet_flag ) {
  }
 }
 
-msleep(msec) { /* wait milli second */
-  unsigned long i;
-  i = msec*1000;
-  usleep(i);
-  return;
+msleep(int msec) { /* wait milli second */
+ struct timeval timeout;
+ timeout.tv_sec = msec / 1000;
+ timeout.tv_usec = (msec % 1000) * 1000;
+ if (select(0, (fd_set *) 0, (fd_set *) 0, (fd_set *) 0, &timeout) < 0) {
+   perror("msleep");
+   return -1;
+ }
 }
